FUNCTION Horoshenkov_Czz, lag, sigmaw,Lo,var

; Given the lag and parameters sigmaw, Lo, and var, this routine returns 
; the autocovariance corresponding to W(rho) (times the variance) in 
; Horoshenkov, et al, 2013. The pattern of surface waves in a shallow free surface flow. 
; JGR Earth Surf, 118, 1864-1876.

; NOTE: Horoshenkov has two versions of the W(rho) equation: The abstract and Section 4 versions have a 1/2 in
; the exponential term; the Eq. (4) version does not.  I don't know which is correct.  This routine uses
; the version with the 1/2 in the exponent.

; Input:
;   lag = the spatial lag in units of meters where Czz is to be returned
;       lag can be a single number or a 1D array with pos or neg values
;   sigmaw = the correlation radius, in m, e.g. from Horoshenkov Table 3
;   Lo = the characteristic length, in m, e.g. from Horoshenkov Table 3
;   var = the variance of the water surface, in m^2, which converts the H autocorrelation to an autocovariance
;   Note: sigmaw, Lo, and var must be passed via a common block if the function is called by a numerical
;         integration routine such as qromb, which requires as input a function routine
;         with just one argument for the abscissa

; Output:
;   Czz(lag) = the autocovariance at the given lag values, units of m^2

; Written 07 April 2016 by Curtis Mobley

;**********************

Czz = var*exp(-0.5*lag^2/sigmaw^2)*cos(2.0*!pi*lag/Lo)

RETURN, Czz

END