pro LogLabels, axmin,axmax, nticks,ticklabs

; This routine creates axis labels of the form 10^x for log axes

; Input:
;   axmin = the min axis value for the plot; equal to a power of 10
;   axmax = the max axis value for the plot; equal to a power of 10 > axmin

; Output:
;   nticks = the number of axis tick marks to be labelled, one per decade
;   ticklabs = the string array of nticks values us use for the axes labels

; Example Useage:
;xmin = 0.01
;xmax = 1000.0
;LogLabels, xmin,xmax, nticksx, xticklabs
;cgplot,x,y, /Xlog,$
;   xrange=[xmin,xmax], xtitle = 'x values', /xstyle, xticks = nticksx-1, xtickname=xticklabs, $
;   yrange=[0,50], ytitle = ylabel

; note: the xticks keyword is the number of axis INTERVALS to be labeled
;------------

axminlog = fix(alog10(axmin))
axmaxlog = fix(alog10(axmax))
nticks = axmaxlog - axminlog + 1
ticklabs = strarr(nticks)
for i=0,nticks-1 do ticklabs(i) = '10!u' + string(axminlog+i,format='(i2)') + '!n'

RETURN
END
