function plotsym, circle=circle, triangle=triangle, diamond=diamond, $
                  box=box, hline=hline, vline=vline, $
                  star4=star4, star5=star5, star6=star6,  del=del, $
                  angle=angle, scale=scale, _extra=extra
;+
; NAME:
;  plotsym
; PURPOSE:
;  function to make plotting symbols much easier.
; Usage:
;   plot,x,y,psym=plotsym(/circle,scale=2,/fill)
;
; CATEGORY:
;   Graphics.
;
; Keywords:
;  circle =  circle symbol
;  triangle = triangle symbol
;  diamond = diamond symbol
;  box = box (square) symbol
;  hline = horizontal line symbol
;  vline = vertical line symbol
;  star4 = 4 pointed star symbol
;  star5 = 5 pointed star symbol
;  star6 = 6 pointed star symbol
;  del = upside down triangle symbol
;  scale = scales the symbol
;  angle = angle the symbol should be rotated
;  _extra = extra keywords for usersym.  These are thick, color and fill
;
; Written by:
; Ronn Kling
; Kling Research and Software
; 7038 Westmoreland Dr.
; Warrenton, VA 20187
; ronn@rlkling.com

; Original software modified December 25, 2002 by
; Curtis Mobley
; Sequoia Scientific, Inc.
; curtis.mobley@sequoiasci.com
; The following changes were made:
; * added star4, star5, star6, hline, vline, and del symbols
; * redefined all symbols in terms of (x,y) coordinates
; * rescaled all symbols for visual appearance
;
; LICENSE
;
; This software is OSI Certified Open Source Software.
; OSI Certified is a certification mark of the Open Source Initiative.
;
; Copyright � 1997-2001 Kling Research and Software.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
; For more information on Open Source Software, visit the Open Source
; web site: http://www.opensource.org.
;-


if not keyword_set(scale) then scale=1.0
if not keyword_set(angle) then angle=0.0

if keyword_set(circle) then begin
  theta = findgen(37)*10.0
  ss = 0.75
  x = ss*cos(theta * !dtor) * scale
  y = ss*sin(theta * !dtor) * scale
endif else if keyword_set(triangle) then begin
  y1 = 0.5*tan(30.0*!dtor)
  ss = 2.0*cos(30.0*!dtor)
  x = ss*[ 0.5, 0.0,-0.5,  0.5] * scale
  y = ss*[-y1,2.0*y1,-y1,-y1] * scale
endif else if keyword_set(diamond) then begin
  x = [1,0,-1,0,1] * scale
  y = [0,1,0,-1,0] * scale
endif else if keyword_set(box) then begin
  ss = 1.0/cos(45.0 * !dtor)
  x = ss*[0.5,-0.5,-0.5, 0.5,0.5] * scale
  y = ss*[0.5, 0.5,-0.5,-0.5,0.5] * scale
endif else if keyword_set(hline) then begin
  xs = 1.0
  ys = 0.05
  x = [xs,-xs,-xs,xs,xs] * scale
  y = [ys,ys,-ys,-ys,ys] * scale
endif else if keyword_set(vline) then begin
  xs = 0.05
  ys = 1.0
  x = [xs,-xs,-xs,xs,xs] * scale
  y = [ys,ys,-ys,-ys,ys] * scale
endif else if keyword_set(star4) then begin
  x1 = 0.2
  x = [1,x1,0,-x1,-1,-x1,0,x1,1] * scale
  y = [0,x1,1,x1,0,-x1,-1,-x1,0] * scale
endif else if keyword_set(star5) then begin
  c18 = cos(18.0 * !dtor)
  c54 = 0.382*cos(54.0 * !dtor)
  c306 = cos(306.0 * !dtor)
  c342 = 0.382*cos(342.0 * !dtor)
  s18 = sin(18.0 * !dtor)
  s54 = 0.382*sin(54.0 * !dtor)
  s306 = sin(306.0 * !dtor)
  s342 = 0.382*sin(342.0 * !dtor)
  x = [c18,c54,0,-c54,-c18,-c342,-c306, 0,  c306,c342,c18] * scale
  y = [s18,s54,1, s54, s18, s342, s306,-0.5,s306,s342,s18] * scale
endif else if keyword_set(star6) then begin
  ss = 0.15  ; 0.15 gives a nice snowflake; 0.33 gives a Star of David
  s60 = ss*sin(60.0 * !dtor)
  c60 = ss*cos(60.0 * !dtor)
  y1 = 0.5*tan(30.0*!dtor)
  s2 = 1.0/tan(30.0*!dtor)
  x = s2*[0.5,ss ,0.5,c60,0.0   ,-c60,-0.5,-ss,-0.5,-c60,0.0    , c60,0.5] * scale
  y = s2*[-y1,0.0,y1 ,s60,2.0*y1, s60, y1 ,0.0,-y1 ,-s60,-2.0*y1,-s60,-y1] * scale
endif else if keyword_set(del) then begin
  y1 = 0.5*tan(30.0*!dtor)
  ss = 2.0*cos(30.0*!dtor)
  x = ss*[0.5,-0.5,0.0,0.5] * scale
  y = ss*[y1,y1,-2.0*y1,y1] * scale
endif

usersym, x,y, _extra=extra
return,8
end